//
//  ViewController.m
//  treemap
//
//  Created by Jacob Gundersen on 11/12/11.
//  Copyright (c) 2011 Interrobang Software LLC. All rights reserved.
//

#import "ViewController.h"

@implementation ViewController

@synthesize data, sortedKeys;

#pragma mark -

- (void)updateCell:(TreemapViewCell *)cell forIndex:(NSInteger)index {
	NSString *key = [self.sortedKeys objectAtIndex:index];
	NSNumber *val = [self.data valueForKey:key];
	cell.textLabel.text = key;
	cell.valueLabel.text = [val stringValue];
    
    //cell.backgroundColor = [UIColor colorWithHue:(float)index / (self.sortedKeys.count + 3)
									  saturation:1 brightness:0.75 alpha:1];
                            
    NSString *lexiClass = [[tags objectForKey:key] objectForKey:@"tense"];
    UIColor *cellBack;
    
    if ([lexiClass isEqualToString:@"Noun"]) {
        cellBack = [UIColor colorWithRed:0.5 green:0.0 blue:(float)index/(self.sortedKeys.count / 2) alpha:1.0];
    } else if ([lexiClass isEqualToString:@"Verb"]) {
        cellBack = [UIColor colorWithRed:0.5 green:0.5 blue:(float)index/(self.sortedKeys.count / .5) alpha:1.0];
    } else if ([lexiClass isEqualToString:@"PersonalName"]) {
        cellBack = [UIColor colorWithRed:(float)index/(self.sortedKeys.count / 2) green:0.5 blue:0.5 alpha:1.0];
    } else {
        cellBack = [UIColor colorWithRed:0.3 green:(float)index/(self.sortedKeys.count / .5) blue:0.3 alpha:1.0];
    }
    
    cell.backgroundColor = cellBack;
}

#pragma mark -
#pragma mark TreemapView delegate

- (void)treemapView:(TreemapView *)treemapView tapped:(NSInteger)index {
    TreemapViewCell *cell = (TreemapViewCell *)[self.view.subviews objectAtIndex:index];
    NSString *searchS = cell.textLabel.text;
    [self searchTwitterWithString:searchS andNumResults:200];
    
	[(TreemapView *)self.view reloadData];
}

#pragma mark -
#pragma mark TreemapView data source



- (NSArray *)valuesForTreemapView:(TreemapView *)treemapView {
	NSMutableArray *values = [NSMutableArray arrayWithCapacity:self.sortedKeys.count];
	for (NSString *key in self.sortedKeys) {
		[values addObject:[self.data valueForKey:key]];
	}
	return values;
}

- (TreemapViewCell *)treemapView:(TreemapView *)treemapView cellForIndex:(NSInteger)index forRect:(CGRect)rect {
	TreemapViewCell *cell = [[TreemapViewCell alloc] initWithFrame:rect];
	[self updateCell:cell forIndex:index];
	return cell;
}

- (void)treemapView:(TreemapView *)treemapView updateCell:(TreemapViewCell *)cell forIndex:(NSInteger)index forRect:(CGRect)rect {
	[self updateCell:cell forIndex:index];
}

#pragma mark -

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation {
	return YES;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation 
// A this point, our view orientation is set to the new orientation.
{
	TreemapView *tree = (TreemapView *)self.view;
	[tree setNeedsLayout];
}

- (void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
}

- (void)viewDidLoad {
	[super viewDidLoad];
	
	//NSString *path = [[NSBundle mainBundle] pathForResource:@"data" ofType:@"plist"];
	//self.data = [NSMutableDictionary dictionaryWithContentsOfFile:path];
	
	/*	This is actually where I got the sample data.
     NSError *error = nil;
     NSString *path = [NSString stringWithFormat:@"http://openstates.sunlightlabs.com/api/v1/subject_counts/tx/82/upper/?apikey=%@", apiKey];
     NSURL *url = [NSURL URLWithString:path];
     NSData *jsonData = [NSData dataWithContentsOfURL:url];
     self.data = [jsonData mutableObjectFromJSONDataWithParseOptions:JKParseOptionLooseUnicode error:&error];
	 */
	
	//self.sortedKeys = [[self.data allKeys] sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
    [self searchTwitterWithString:@"joe" andNumResults:200];
}

-(void)searchTwitterWithString:(NSString *)str andNumResults:(int)num {
    NSString *urlString = @"http://search.twitter.com/search.json";
    NSDictionary *param = [NSDictionary dictionaryWithObjectsAndKeys:
                           str, @"q", 
                           [NSString stringWithFormat:@"%d", num], @"rpp", 
                           @"en", @"lang", 
                           nil];
    TWRequest *tr = [[TWRequest alloc] initWithURL:[NSURL URLWithString:urlString] 
                                        parameters:param 
                                     requestMethod:TWRequestMethodGET];
    
    [tr performRequestWithHandler:^(NSData *responseData, NSHTTPURLResponse *urlResponse, NSError *error) {
        NSDictionary *dict = [NSJSONSerialization JSONObjectWithData:responseData options:0 error:&error];
        if (error) {
            NSLog(@"%@", error.localizedDescription);
        } else {
            NSString *n = @"";
            for (NSDictionary *d in [dict objectForKey:@"results"]) {
                n = [n stringByAppendingFormat:@" %@", [d objectForKey:@"text"]];
            }
            NSDictionary *regexStrings = [NSDictionary dictionaryWithObjectsAndKeys:
                                     @"http.+?[ \n\"]", @"links", 
                                     @"@.+?[ \n]", @"mentions", 
                                     @"#.+?[ \n]", @"hashtags", nil];
            for (NSString *key in [regexStrings allKeys]) {
                NSRegularExpression *regex = [[NSRegularExpression alloc] initWithPattern:[regexStrings objectForKey:key] options:0 error:nil];
                n = [regex stringByReplacingMatchesInString:n options:0 range:NSMakeRange(0, [n length]) withTemplate:@""];
            }
            
            NSLinguisticTagger *tagger = [[NSLinguisticTagger alloc] 
                                          initWithTagSchemes:[NSArray arrayWithObjects:
                                           NSLinguisticTagSchemeNameTypeOrLexicalClass, NSLinguisticTagSchemeLemma, nil]
                                                    options:(NSLinguisticTaggerOmitWhitespace | NSLinguisticTaggerOmitPunctuation)];
            [tagger setString:n];
            
            tags = [NSMutableDictionary dictionary];
            
            NSArray *stopwords = [NSArray arrayWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"stopwords" ofType:@"plist" ]];
            
            [tagger enumerateTagsInRange:NSMakeRange(0, [n length]) 
                                  scheme:NSLinguisticTagSchemeLemma 
                                 options:(NSLinguisticTaggerOmitWhitespace | NSLinguisticTaggerOmitPunctuation) 
                              usingBlock:^(NSString *tag, NSRange tokenRange, NSRange sentenceRange, BOOL *stop) {
                                  NSString *tag2 = [tagger tagAtIndex:tokenRange.location scheme:NSLinguisticTagSchemeNameTypeOrLexicalClass tokenRange:NULL sentenceRange:NULL];
                                  NSLog(@"%@, %@, %@", [n substringWithRange:tokenRange], tag, tag2);
                                  if (![stopwords containsObject:tag] && tag != nil) {
                                      if (![[tags allKeys] containsObject:tag]) {
                                          NSMutableDictionary *tagDict = [NSMutableDictionary dictionaryWithObjectsAndKeys:tag2, @"tense", [NSNumber numberWithInt:1], @"count", nil];
                                          [tags setObject:tagDict forKey:tag];
                                      } else {
                                          NSMutableDictionary *tagDict = [tags objectForKey:tag];
                                          [tagDict setObject:[NSNumber numberWithInt:([[tagDict objectForKey:@"count"] intValue] + 1)] forKey:@"count"];
                                      }
                                  }
                                  
            }];
            
            NSMutableDictionary *mut = [NSMutableDictionary dictionary];
            for (NSString *k in tags) {
                if ([[[tags objectForKey:k] objectForKey:@"count"] intValue] > 1 && ![k isEqualToString:str]) {
                    [mut setObject:[[tags objectForKey:k] objectForKey:@"count"] forKey:k];
                }
            }
            data = mut;
            sortedKeys = [[data allKeys] sortedArrayUsingSelector:@selector(caseInsensitiveCompare:)];
            [(TreemapView *)self.view createNodes];
           
        }
    }];
}
     
- (void)viewDidUnload {
	[super viewDidUnload];
	
	self.sortedKeys = nil;
	self.data = nil;
}


@end
